@@ -26,15 +26,16 @@
 #include "ac_config.h"
 
 
-#ifdef HAVE_LADCCA
-	#include <ladcca/ladcca.h>
-	extern cca_client_t * global_cca_client;
+#ifdef HAVE_LASH
+    #include <lash/lash.h>
+    extern lash_client_t * global_lash_client;
 #endif
 
+
 #define JACK_RACK_LOGO_FILE "jack-rack-logo.png"
 #define JACK_RACK_ICON_FILE "jack-rack-icon.png"
 #define JACK_RACK_CHANNELS_ICON_FILE "gnome-mixer-small.png"
-#define JACK_RACK_URL "http://pkl.net/~node/jack-rack.html"
+#define JACK_RACK_URL "http://jack-rack.sourceforge.net/"
 
 
 #define set_string_property(property, value) \
@@ -46,20 +47,14 @@
     (property) = g_strdup (value); \
   else \
     (property) = NULL;
-    
-//#ifdef HAVE_GNOME
-#if 0
-	#include <libgnome/gnome-i18n.h>
-#else
-	#ifdef ENABLE_NLS
-		#include <libintl.h>
-		#define _(x) gettext(x)
-	#else
-		#define _(x) x
-	#endif
 
-	#define N_(x) x
+#ifdef ENABLE_NLS
+#       include <libintl.h>
+#       define _(x) dgettext(PACKAGE, x)
+#else
+#       define _(x) x
 #endif
+#define N_(x) x
 
 extern struct _ui *global_ui;
 extern gboolean   connect_inputs;
@@ -68,4 +63,17 @@ extern gboolean   time_runs;
 extern GString    *client_name;
 extern GString    *initial_filename;
 
+typedef enum MIDI_CONTROL_TYPE 
+{
+   LADSPA_CONTROL,
+   WET_DRY_CONTROL,
+   PLUGIN_ENABLE_CONTROL
+} midi_control_type_t;
+
+#define COPYRIGHT_MESSAGE \
+	"Copyright (C) 2002, 2003 Robert Ham <node@users.sourceforge.net>\n" \
+	"Copyright (C) 2005, 2006 Leslie P. Polzer <leslie.polzer@gmx.net>\n" \
+	"Copyright (C) 2006, 2007 Adam Sampson <ats@offog.org>"
+
 #endif /* __JLH_GLOBALS_H__ */
+
