@@ -32,10 +32,6 @@
 #include <signal.h>
 #include <locale.h>
 
-#ifdef HAVE_LADCCA
-#include <ladcca/ladcca.h>
-#endif
-
 #ifdef HAVE_GNOME
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
@@ -57,15 +53,15 @@ gboolean time_runs = TRUE;
 GString  *client_name = NULL;
 GString  *initial_filename = NULL;
 
-#ifdef HAVE_LADCCA
-cca_client_t * global_cca_client;
+#ifdef HAVE_LASH
+lash_client_t * global_lash_client;
 #endif
 
 #define CLIENT_NAME_BASE      "JACK Rack"
 
-void print_help () {
+void print_help (void) {
   printf("%s version %s\n", PACKAGE_NAME, PACKAGE_VERSION);
-  printf("Copyright (C) 2002, 2003 Robert Ham (node@users.sourceforge.net)\n");
+  printf("%s\n", COPYRIGHT_MESSAGE);
   printf("\n");
   printf("This program comes with ABSOLUTELY NO WARRANTY.  You are licensed to use it\n");
   printf("under the terms of the GNU General Public License, version 2 or later.  See\n");
@@ -76,8 +72,8 @@ void print_help () {
 #ifdef HAVE_ALSA
   printf(  "  ALSA %s\n", ALSA_VERSION);
 #endif
-#ifdef HAVE_LADCCA
-  printf(  "  LADCCA %s\n", LADCCA_VERSION);
+#ifdef HAVE_LASH
+  printf(  "  LASH %s\n", LASH_VERSION);
 #endif
 #ifdef HAVE_XML
   printf(  "  libxml2 %s\n", XML_VERSION);
@@ -86,7 +82,7 @@ void print_help () {
   printf(  "  GNOME %s\n", GNOME_VERSION);
 #endif
   printf("\n");
-  printf("Usage: jack-rack [OPTION]...");
+  printf(_("Usage: jack-rack [OPTION]..."));
 #ifdef HAVE_XML
   printf(" [file]\n");
 #else
@@ -124,15 +120,15 @@ int main (int argc, char ** argv) {
     { 0, 0, 0, 0 }
   };
 
-#ifdef HAVE_LADCCA
-  cca_args_t * cca_args;
-  cca_event_t * cca_event;
+#ifdef HAVE_LASH
+  lash_args_t * lash_args;
+  lash_event_t * lash_event;
 #endif  
 
+  gtk_set_locale ();
 #ifdef ENABLE_NLS
   bindtextdomain (PACKAGE, LOCALEDIR);
-  textdomain(PACKAGE);
-  bind_textdomain_codeset(PACKAGE, "UTF-8");
+  bind_textdomain_codeset (PACKAGE, "UTF-8");
 #endif
   
   /* not using gnome popt */
@@ -147,11 +143,10 @@ int main (int argc, char ** argv) {
     }
 
 
-#ifdef HAVE_LADCCA
-  cca_args = cca_extract_args (&argc, &argv);
+#ifdef HAVE_LASH
+  lash_args = lash_extract_args (&argc, &argv);
 #endif  
 
-  gtk_set_locale ();
   gtk_init (&argc, &argv);
 
 #ifdef HAVE_GNOME
@@ -196,7 +191,7 @@ int main (int argc, char ** argv) {
         channels = atof (optarg);
         if (channels < 1)
           {
-            fprintf (stderr, _("there must be at least one channel\n"));
+            fprintf (stderr, _("There must be at least one channel\n"));
             exit (EXIT_FAILURE);
           }
         break;
@@ -218,7 +213,7 @@ int main (int argc, char ** argv) {
   {
     if (argc - optind > 1)
     {
-      fprintf ( stderr, "Please specify only one file to open");
+      fprintf (stderr, _("Please specify only one file to open"));
       exit(EXIT_FAILURE);
     }
 
@@ -226,19 +221,19 @@ int main (int argc, char ** argv) {
   }
 #endif /* HAVE_XML */
 
-#ifdef HAVE_LADCCA
+#ifdef HAVE_LASH
   {
-    int flags = CCA_Config_File;
-    global_cca_client = cca_init (cca_args, PACKAGE_NAME, flags, CCA_PROTOCOL (2,0));
+    int flags = LASH_Config_File;
+    global_lash_client = lash_init (lash_args, PACKAGE_NAME, flags, LASH_PROTOCOL (2,0));
   }
 
-  if (global_cca_client)
+  if (global_lash_client)
     {
-      cca_event = cca_event_new_with_type (CCA_Client_Name);
-      cca_event_set_string (cca_event, client_name->str);
-      cca_send_event (global_cca_client, cca_event);
+      lash_event = lash_event_new_with_type (LASH_Client_Name);
+      lash_event_set_string (lash_event, client_name->str);
+      lash_send_event (global_lash_client, lash_event);
     }
-#endif /* HAVE_LADCCA */
+#endif /* HAVE_LASH */
 
 #ifdef HAVE_XML
   xmlSetCompressMode (XML_COMPRESSION_LEVEL);
