@@ -79,6 +79,7 @@ plugin_slot_show_wet_dry_controls (plugin_slot_t * plugin_slot)
     gtk_widget_hide (plugin_slot->wet_dry_controls->control_box);
 }
 
+
 void
 plugin_slot_show_controls (plugin_slot_t * plugin_slot, guint copy_to_show)
 {
@@ -129,6 +130,9 @@ plugin_slot_show_controls (plugin_slot_t * plugin_slot, guint copy_to_show)
 }
 
 
+/**
+ * Adjust GUI controls to reflect plugin settings
+ */
 void
 plugin_slot_set_port_controls (plugin_slot_t *plugin_slot,
                                port_controls_t *port_controls,
@@ -151,6 +155,21 @@ plugin_slot_set_port_controls (plugin_slot_t *plugin_slot,
               port_controls_block_float_callback (port_controls, copy);
             gtk_range_set_value (GTK_RANGE (port_controls->controls[copy].control),
                                  logval);
+
+  /* XXX <EXPERIMENTAL> */
+  /* possibly set our peers */
+  if (port_controls->plugin_slot->plugin->copies > 1
+      && port_controls->locked)
+    {
+      guint i;
+      for (i = 0; i < port_controls->plugin_slot->plugin->copies; i++)
+        {
+          if (i != copy)
+            gtk_range_set_value (GTK_RANGE(port_controls->controls[i].control),
+                                 gtk_range_get_value (GTK_RANGE (port_controls->controls[copy].control)));
+        }
+    }
+  /* </EXPERIMENTAL> */
             if (block_callbacks)
               port_controls_unblock_float_callback (port_controls, copy);
               
@@ -169,6 +188,37 @@ plugin_slot_set_port_controls (plugin_slot_t *plugin_slot,
               port_controls_unblock_int_callback (port_controls, copy);
           break;
 
+        case JR_CTRL_POINTS: {
+          GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (port_controls->controls[copy].control));
+          GtkTreeIter iter;
+          gboolean is_first = TRUE, valid;
+          GtkTreeIter nearest_iter;
+          gfloat nearest_diff;
+
+          if (block_callbacks)
+              port_controls_block_points_callback (port_controls, copy);
+
+          /* Select the nearest point to the current value. */
+          gtk_tree_model_get_iter_first (model, &iter);
+          do {
+            float this_diff;
+            gtk_tree_model_get (model, &iter, 1, &this_diff, -1);
+            this_diff = fabs (this_diff - value);
+            if (is_first || this_diff < nearest_diff) {
+              nearest_iter = iter;
+              nearest_diff = this_diff;
+            }
+            is_first = FALSE;
+            valid = gtk_tree_model_iter_next (model, &iter);
+          } while (valid);
+          gtk_combo_box_set_active_iter
+            (GTK_COMBO_BOX (port_controls->controls[copy].control), &nearest_iter);
+
+          if (block_callbacks)
+              port_controls_unblock_points_callback (port_controls, copy);
+          break;
+        }
+
         case JR_CTRL_BOOL:
           if (block_callbacks)
               port_controls_block_bool_callback (port_controls, copy);
@@ -181,6 +231,7 @@ plugin_slot_set_port_controls (plugin_slot_t *plugin_slot,
     }
 }
 
+
 void
 plugin_slot_set_wet_dry_controls (plugin_slot_t *plugin_slot, gboolean block_callbacks)
 {
@@ -201,6 +252,7 @@ plugin_slot_set_wet_dry_controls (plugin_slot_t *plugin_slot, gboolean block_cal
                                   settings_get_wet_dry_locked (plugin_slot->settings));
 }
 
+
 static void
 plugin_slot_set_controls (plugin_slot_t * plugin_slot, settings_t * settings)
 {
@@ -433,23 +485,35 @@ plugin_slot_new     (jack_rack_t * jack_rack, plugin_t * plugin, saved_plugin_t
 #ifdef HAVE_ALSA
     GSList *list;
     midi_control_t *midi_ctrl;
-    midi_control_t *new_midi_ctrl;
+    midi_control_t *new_midi_ctrl = NULL;
     
     for (list = saved_plugin->midi_controls; list; list = g_slist_next (list))
       {
         midi_ctrl = list->data;
         
-        if (midi_ctrl->ladspa_control)
-          new_midi_ctrl =
-            ladspa_midi_control_new (plugin_slot,
-                                     midi_ctrl->control.ladspa.copy,
-                                     midi_ctrl->control.ladspa.control);
-        else
-          new_midi_ctrl =
-            wet_dry_midi_control_new (plugin_slot, midi_ctrl->control.wet_dry.channel);
+	switch(midi_ctrl->ctrl_type)
+	  {
+	  case LADSPA_CONTROL:
+	    new_midi_ctrl =
+	      ladspa_midi_control_new (plugin_slot,
+				       midi_ctrl->control.ladspa.copy,
+				       midi_ctrl->control.ladspa.control);
+	    break;
+	  case WET_DRY_CONTROL:
+	    new_midi_ctrl =
+	      wet_dry_midi_control_new (plugin_slot, 
+					midi_ctrl->control.wet_dry.channel);
+	    break;
+	  case PLUGIN_ENABLE_CONTROL:
+	    new_midi_ctrl =
+	      toggle_midi_control_new (plugin_slot);
+	    break;
+	  }
         
         midi_control_set_midi_channel (new_midi_ctrl, midi_ctrl->midi_channel);
         midi_control_set_midi_param   (new_midi_ctrl, midi_ctrl->midi_param);
+        midi_control_set_min_value    (new_midi_ctrl, midi_ctrl->min);
+        midi_control_set_max_value    (new_midi_ctrl, midi_ctrl->max);
         
         plugin_slot_add_midi_control (plugin_slot, new_midi_ctrl);
         
@@ -586,7 +650,7 @@ plugin_slot_set_wet_dry_locked (plugin_slot_t *plugin_slot, gboolean locked)
     {
       midi_ctrl = (midi_control_t *) list->data;
       
-      if (!midi_ctrl->ladspa_control)
+      if (midi_ctrl->ctrl_type == WET_DRY_CONTROL)
         midi_control_set_locked (midi_ctrl, locked);
     }
 #endif
@@ -617,9 +681,11 @@ plugin_slot_set_lock_all (plugin_slot_t *plugin_slot, gboolean lock_all, guint l
     {
       midi_ctrl = (midi_control_t *) list->data;
       
-      if (midi_ctrl->ladspa_control)
-        midi_control_set_locked (midi_ctrl,
-          lock_all ? TRUE : plugin_slot->port_controls[midi_ctrl->control.ladspa.control].locked);
+      if (midi_ctrl->ctrl_type == LADSPA_CONTROL)
+	    {
+	      midi_control_set_locked (midi_ctrl,
+				   lock_all ? TRUE : plugin_slot->port_controls[midi_ctrl->control.ladspa.control].locked);
+	    }
     }
 #endif /* HAVE_ALSA */
   
