@@ -36,29 +36,67 @@
 #include "jack_rack.h"
 #include "ui.h"
 #include "ui_callbacks.h"
+#include "util.h"
+
 
 #define USEC_PER_SEC         1000000
 #define MSEC_PER_SEC         1000
 #define TIME_RUN_SKIP_COUNT  5
 
+
+extern void default_status_handler (void*, const char*, ...);
+extern void default_error_handler (void*, error_severity_t, const char*, ...);
+
 jack_nframes_t sample_rate;
 jack_nframes_t buffer_size;
 
 static char * jack_client_name;
 
-int process_control_messages (process_info_t * procinfo) {
+static status_callback _update_status = default_status_handler;
+static error_callback _display_error = default_error_handler;
+
+static void* _status_cb_data = NULL;
+static void* _error_cb_data = NULL;
+
+
+/**
+ * Set the function to call when status changes 
+ *
+ * @param data The data to pass to the function
+ */
+void process_set_status_cb ( void* data, status_callback cb )
+{
+	_update_status = cb;
+        _status_cb_data = data;
+}
+
+
+/**
+ * Set the function to call when an error occurs
+ *
+ * @see process_set_status_cb
+ */
+void process_set_error_cb ( void* data, error_callback cb )
+{
+	_display_error = cb;
+        _error_cb_data = data;
+}
+
+
+int process_control_messages (process_info_t * procinfo)
+{
   static int quitting = 0;
   ctrlmsg_t ctrlmsg;
   int err = 0;
   
-  if (quitting) return quitting;
+  if (quitting)
+	  return quitting;
   
   while (lff_read (procinfo->ui_to_process, &ctrlmsg) == 0)
     {
   
     switch (ctrlmsg.type)
       {
-    
       /* add a link to the end of the plugin chain */
       case CTRLMSG_ADD:
         process_add_plugin (procinfo, ctrlmsg.data.add.plugin);
@@ -118,11 +156,14 @@ int process_control_messages (process_info_t * procinfo) {
         quitting = 1;
         err = lff_write (procinfo->process_to_ui, &ctrlmsg);
         return 1;
+
+      default:
+        break;
       }
     
     if (err)
       {
-        fprintf (stderr, "%s: gui fifo is out of space\n", __FUNCTION__);
+        _display_error ( _error_cb_data, E_ERROR, _("%s: GUI FIFO is out of space\n"), __FUNCTION__ );
         return err;
       }
       
@@ -132,7 +173,8 @@ int process_control_messages (process_info_t * procinfo) {
 }
 
 /** process messages for plugins' control ports */
-void process_control_port_messages (process_info_t * procinfo) {
+void process_control_port_messages (process_info_t * procinfo)
+{
   plugin_t * plugin;
   unsigned long control;
   unsigned long channel;
@@ -167,7 +209,8 @@ void process_control_port_messages (process_info_t * procinfo) {
     }
 }
 
-int get_jack_buffers (process_info_t * procinfo, jack_nframes_t frames) {
+int get_jack_buffers (process_info_t * procinfo, jack_nframes_t frames)
+{
   unsigned long channel;
   
   for (channel = 0; channel < procinfo->channels; channel++)
@@ -175,14 +218,14 @@ int get_jack_buffers (process_info_t * procinfo, jack_nframes_t frames) {
       procinfo->jack_input_buffers[channel] = jack_port_get_buffer (procinfo->jack_input_ports[channel], frames);
       if (!procinfo->jack_input_buffers[channel])
         {
-          fprintf (stderr, "%s: no jack buffer for input port %ld\n", __FUNCTION__, channel);
+          fprintf (stderr, _("%s: no JACK buffer for input port %ld\n"), __FUNCTION__, channel);
           return 1;
         }
 
       procinfo->jack_output_buffers[channel] = jack_port_get_buffer (procinfo->jack_output_ports[channel], frames);
       if (!procinfo->jack_output_buffers[channel])
         {
-          fprintf (stderr, "%s: no jack buffer for output port %ld\n", __FUNCTION__, channel);
+          fprintf (stderr, _("%s: no JACK buffer for output port %ld\n"), __FUNCTION__, channel);
           return 1;
         }
     }
@@ -190,6 +233,7 @@ int get_jack_buffers (process_info_t * procinfo, jack_nframes_t frames) {
   return 0;
 }
 
+
 plugin_t *
 get_first_enabled_plugin (process_info_t * procinfo)
 {
@@ -207,6 +251,7 @@ get_first_enabled_plugin (process_info_t * procinfo)
   return NULL;
 }
 
+
 plugin_t *
 get_last_enabled_plugin (process_info_t * procinfo)
 {
@@ -224,13 +269,13 @@ get_last_enabled_plugin (process_info_t * procinfo)
   return NULL;
 }
 
+
 void
 connect_chain (process_info_t * procinfo, jack_nframes_t frames)
 {
   plugin_t * first_enabled, * last_enabled, * plugin;
   gint copy;
   unsigned long channel;
-  unsigned long rack_channel;
   if (!procinfo->chain) return;
   
   first_enabled = get_first_enabled_plugin (procinfo);
@@ -278,6 +323,7 @@ connect_chain (process_info_t * procinfo, jack_nframes_t frames)
   plugin_connect_input_ports (first_enabled, procinfo->jack_input_buffers);
 }
 
+
 void
 process_chain (process_info_t * procinfo, jack_nframes_t frames)
 {
@@ -336,7 +382,8 @@ process_chain (process_info_t * procinfo, jack_nframes_t frames)
               for (i = 0; i < frames; i++)
                 {
                   plugin->audio_output_memory[channel][i] *= plugin->wet_dry_values[channel];
-                  plugin->audio_output_memory[channel][i] += plugin->audio_input_memory[channel][i] * (1.0 - plugin->wet_dry_values[channel]);
+                  plugin->audio_output_memory[channel][i] += plugin->audio_input_memory[channel][i]
+			  * (1.0 - plugin->wet_dry_values[channel]);
                 }
           
           if (plugin == last_enabled)
@@ -361,15 +408,16 @@ process_chain (process_info_t * procinfo, jack_nframes_t frames)
   
 }
 
+
 int process (jack_nframes_t frames, void * data) {
   int err, quitting;
   process_info_t * procinfo;
   
   procinfo = (process_info_t *) data;
-  
+
   if (!procinfo)
     {
-      fprintf (stderr, "%s: no process_info from jack!\n", __FUNCTION__);
+      _display_error ( _error_cb_data, E_WARNING, _("%s: no process_info from JACK!\n"), __FUNCTION__ );
       return 1;
     }
   
@@ -383,7 +431,7 @@ int process (jack_nframes_t frames, void * data) {
   err = get_jack_buffers (procinfo, frames);
   if (err)
     {
-      fprintf(stderr, "%s: failed to get jack ports, not processing\n", __FUNCTION__);
+      _display_error ( _error_cb_data, E_WARNING, _("%s: failed to get JACK ports, not processing\n"), __FUNCTION__ );
       return 0;
     }
   
@@ -396,32 +444,27 @@ int process (jack_nframes_t frames, void * data) {
 
 
 
-/*******************************************
- ************** non RT stuff ***************
- *******************************************/
- 
+/****
+ * Non-RT stuff starts here
+ ****/
+
 static int
 process_info_connect_jack (process_info_t * procinfo, ui_t * ui)
 {
-  ui_display_splash_text (ui, _("Connecting to JACK server with client name '%s'"), jack_client_name);
+  _update_status ( _("Connecting to JACK server with client name '%s'"), jack_client_name );
 
-  procinfo->jack_client = jack_client_new (jack_client_name);
-
-  if (!procinfo->jack_client)
-    {
-      fprintf (stderr, "%s: could not create jack client; exiting\n", __FUNCTION__);
-      ui_display_error (ui, _("Could not create JACK client\n\nIs the jackd server running?"));
-      return 1;
-    }
-
-  ui_display_splash_text (ui, _("Connected to JACK server"));
+  if (!(procinfo->jack_client = jack_client_new (jack_client_name)))
+        return -1;
+  
+  _update_status ( _status_cb_data, _("Connected to JACK server") );
 
   jack_set_process_callback (procinfo->jack_client, process, procinfo);
-  jack_on_shutdown (procinfo->jack_client, jack_shutdown_cb, ui);
+  jack_on_shutdown (procinfo->jack_client, jack_shutdown_cb, ui); /* FIXME: need a generic callback for this, too */
                                             
   return 0;
 }
 
+
 static void
 process_info_connect_port (process_info_t * procinfo,
                            ui_t * ui,
@@ -449,24 +492,25 @@ process_info_connect_port (process_info_t * procinfo,
         
       full_port_name = g_strdup_printf ("%s:%s", jack_client_name, port_name);
 
-      ui_display_splash_text (ui, _("Connecting ports '%s' and '%s'"), full_port_name, jack_ports[jack_port_index]);
+      _update_status ( _("Connecting port '%s' with '%s'"), full_port_name, jack_ports[jack_port_index] );
 
       err = jack_connect (procinfo->jack_client,
                           in ? jack_ports[jack_port_index] : full_port_name,
                           in ? full_port_name : jack_ports[jack_port_index]);
 
       if (err)
-        fprintf (stderr, "%s: error connecting ports '%s' and '%s'\n",
+        fprintf (stderr, _("%s: error connecting ports '%s' and '%s'\n"),
                  __FUNCTION__, full_port_name, jack_ports[jack_port_index]);
       else
-        ui_display_splash_text (ui, _("Connected ports '%s' and '%s'"), full_port_name, jack_ports[jack_port_index]);
-      
+	      _update_status ( _("Connected port '%s' with '%s'"), full_port_name, jack_ports[jack_port_index] );
+
       free (full_port_name);
     }
   
   free (jack_ports);
 }
 
+
 int
 process_info_set_port_count (process_info_t * procinfo, ui_t * ui, unsigned long port_count)
 {
@@ -476,7 +520,7 @@ process_info_set_port_count (process_info_t * procinfo, ui_t * ui, unsigned long
   gshort in;
   
   if (procinfo->port_count >= port_count)
-      return;
+      return 0;
   
   if (procinfo->port_count == 0)
     {
@@ -488,11 +532,15 @@ process_info_set_port_count (process_info_t * procinfo, ui_t * ui, unsigned long
     }
   else
     {
-      procinfo->jack_input_ports = g_realloc (procinfo->jack_input_ports, sizeof (jack_port_t *) * port_count);
-      procinfo->jack_output_ports = g_realloc (procinfo->jack_output_ports, sizeof (jack_port_t *) * port_count);
+      procinfo->jack_input_ports = g_realloc (procinfo->jack_input_ports,
+                      sizeof (jack_port_t *) * port_count);
+      procinfo->jack_output_ports = g_realloc (procinfo->jack_output_ports,
+                      sizeof (jack_port_t *) * port_count);
 
-      procinfo->jack_input_buffers = g_realloc (procinfo->jack_input_buffers, sizeof (LADSPA_Data *) * port_count);
-      procinfo->jack_output_buffers = g_realloc (procinfo->jack_output_buffers, sizeof (LADSPA_Data *) * port_count);
+      procinfo->jack_input_buffers = g_realloc (procinfo->jack_input_buffers,
+                      sizeof (LADSPA_Data *) * port_count);
+      procinfo->jack_output_buffers = g_realloc (procinfo->jack_output_buffers,
+                      sizeof (LADSPA_Data *) * port_count);
     }
   
   for (i = procinfo->port_count; i < port_count; i++)
@@ -501,8 +549,17 @@ process_info_set_port_count (process_info_t * procinfo, ui_t * ui, unsigned long
       {
         port_name = g_strdup_printf ("%s_%ld", in ? "in" : "out", i + 1);
        
-        ui_display_splash_text (ui, _("Creating %s port %s"), in ? "input" : "output", port_name);
-        
+        if (in)
+          {
+            _update_status (_status_cb_data, _("Creating input port '%s'"),
+                            port_name);
+          }
+        else
+          {
+            _update_status (_status_cb_data, _("Creating output port '%s'"),
+                            port_name);
+          }
+
         port_ptr = (in ? &procinfo->jack_input_ports[i]
                        : &procinfo->jack_output_ports[i]);
         
@@ -514,14 +571,14 @@ process_info_set_port_count (process_info_t * procinfo, ui_t * ui, unsigned long
         
         if (!*port_ptr)
           {
-            fprintf (stderr, "%s: could not register port '%s'; aborting\n",
+            fprintf (stderr, _("%s: could not register port '%s'; aborting\n"),
                      __FUNCTION__, port_name);
-            ui_display_error (ui, "Could not register JACK port '%s'; aborting", port_name);
+            ui_display_error ( _error_cb_data, E_FATAL,
+                               _("Could not register JACK port '%s'; aborting"),
+                               port_name);
             return 1;
           }
 
-        ui_display_splash_text (ui, _("Created %s port %s"), in ? "input" : "output", port_name);
-        
         if ((in && connect_inputs) || (!in && connect_outputs))
           process_info_connect_port (procinfo, ui, in, i, port_name);
         
@@ -534,6 +591,7 @@ process_info_set_port_count (process_info_t * procinfo, ui_t * ui, unsigned long
   return 0;
 }
 
+
 void
 process_info_set_channels (process_info_t * procinfo, ui_t * ui, unsigned long channels)
 {
@@ -541,10 +599,11 @@ process_info_set_channels (process_info_t * procinfo, ui_t * ui, unsigned long c
   procinfo->channels = channels; 
 }
 
+
 process_info_t *
-process_info_new (ui_t * ui, unsigned long rack_channels)
+process_info_new (ui_t* ui, unsigned long rack_channels)
 {
-  process_info_t * procinfo;
+  process_info_t* procinfo;
   int err;
 
   procinfo = g_malloc (sizeof (process_info_t));
@@ -559,32 +618,20 @@ process_info_new (ui_t * ui, unsigned long rack_channels)
   procinfo->time_runs = time_runs;
   
   /* sort out the client name */
-  jack_client_name = strdup (client_name->str);
-  for (err = 0; jack_client_name[err] != '\0'; err++)
-    {
-      if (jack_client_name[err] == ' ')
-        jack_client_name[err] = '_';
-      else if (!isalnum (jack_client_name[err]))
-        { /* shift all the chars up one (to remove the non-alphanumeric char) */
-          int i;
-          for (i = err; jack_client_name[i] != '\0'; i++)
-            jack_client_name[i] = jack_client_name[i + 1];
-        }
-      else if (isupper (jack_client_name[err]))
-        jack_client_name[err] = tolower (jack_client_name[err]);
-    }
+  jack_client_name = g_strdup ( client_name->str );
+  sanitize_client_name ( jack_client_name );
   
-  err = process_info_connect_jack (procinfo, ui);
-  if (err)
-    {
-/*      g_free (procinfo); */
-      return NULL;
-/*      abort (); */
-    }
+  err = process_info_connect_jack (procinfo, ui); 
+  if (err == -1)
+  {
+        g_free (jack_client_name);
+        g_free (procinfo);
+        return NULL;
+  }
   
   err = process_info_set_port_count (procinfo, ui, rack_channels);
   if (err)
-    return NULL;
+        return NULL;
   
   sample_rate = jack_get_sample_rate (procinfo->jack_client);
   buffer_size = jack_get_sample_rate (procinfo->jack_client);
@@ -597,18 +644,19 @@ process_info_new (ui_t * ui, unsigned long rack_channels)
   
   jack_activate (procinfo->jack_client);
 
-#ifdef HAVE_LADCCA
-  /* sort out ladcca stuff */
-  cca_jack_client_name (global_cca_client, jack_client_name);
+#ifdef HAVE_LASH
+  /* sort out LASH stuff */
+  lash_jack_client_name (global_lash_client, jack_client_name);
 #endif
 
   return procinfo;
 }
 
+
 void
-process_info_destroy (process_info_t * procinfo) {
+process_info_destroy (process_info_t * procinfo)
+{
   jack_deactivate (procinfo->jack_client);
   jack_client_close (procinfo->jack_client);
   g_free (procinfo);
 }
-
