@@ -69,25 +69,26 @@ struct _ui
   GtkWidget *       main_window;
   GtkWidget *       plugin_box;
   GtkWidget *       add_menu;
+  GtkWidget *       toolbar_add_menu;
 
 /* all the widgets that can be desensitized */
-  GtkWidget *       add;
+  GtkToolItem *     add;
   GtkWidget *       add_menuitem;
   GtkWidget *       channels_menuitem;
-  GtkWidget *       channels;
+  GtkToolItem *     channels;
 #ifdef HAVE_ALSA
   GtkWidget         *midi_menuitem;
 #endif
   GtkWidget *       new_menuitem;
-  GtkWidget *       new;
+  GtkToolItem *     new;
 #ifdef HAVE_XML
   GtkWidget *       open_menuitem;
-  GtkWidget *       open;
+  GtkToolItem *     open;
 #endif
 
-#ifdef HAVE_LADCCA
-  GtkWidget *       cca_save;
-  GtkWidget *       cca_save_menu_item;
+#ifdef HAVE_LASH
+  GtkToolItem *     lash_save;
+  GtkWidget *       lash_save_menu_item;
 #endif
 
   GtkWidget *       splash_screen;
@@ -100,19 +101,21 @@ struct _ui
 };
 
 ui_t * ui_new     (unsigned long channels);
-void   ui_destroy (ui_t * ui);
+void   ui_destroy (ui_t* ui);
 
-void ui_set_filename (ui_t * ui, const char * filename);
-void ui_set_state    (ui_t * ui, ui_state_t state);
-void ui_set_channels (ui_t * ui, unsigned long channels);
+void ui_set_filename (ui_t* ui, const char* filename);
+void ui_set_state    (ui_t* ui, ui_state_t state);
+void ui_set_channels (ui_t* ui, unsigned long channels);
 
-ui_state_t ui_get_state (ui_t * ui);
+ui_state_t ui_get_state (ui_t* ui);
 
-plugin_t * ui_instantiate_plugin (ui_t * ui, plugin_desc_t * desc);
+plugin_t * ui_instantiate_plugin (ui_t* ui, plugin_desc_t* desc);
 
-void     ui_display_error       (ui_t * ui, const char * format, ...);
-void     ui_display_splash_text (ui_t * ui, const char * format, ...);
-gboolean ui_get_ok              (ui_t * ui, const char * format, ...);
+void     ui_display_error       (void* data, error_severity_t severity, const char* format, ...);
+void     ui_display_splash_text (void* data, const char* format, ...);
+gboolean ui_get_ok              (ui_t* ui, const char* format, ...);
 
+int ui_save_file (ui_t * ui, const char * filename);
+int ui_open_file (ui_t * ui, const char * filename);
 
 #endif /* __JR_UI_H__ */
