@@ -30,10 +30,6 @@
 #include <gtk/gtk.h>
 #include <ladspa.h>
 
-#ifdef HAVE_LADCCA
-#include <ladcca/ladcca.h>
-#endif
-
 #ifdef HAVE_GNOME
 #include <libgnomeui/libgnomeui.h>
 #endif
@@ -45,8 +41,10 @@
 #include "globals.h"
 #include "file.h"
 #include "ui.h"
+#include "process.h"
 #include "wet_dry_controls.h"
 
+
 void
 add_cb (GtkMenuItem * menuitem, gpointer user_data)
 {
@@ -55,11 +53,10 @@ add_cb (GtkMenuItem * menuitem, gpointer user_data)
   
   ui = (ui_t *) user_data;
   
-  /* get the selected plugin out of the gobject data thing */
   desc = g_object_get_data (G_OBJECT(menuitem), "jack-rack-plugin-desc");
   if (!desc)
     {
-      fprintf (stderr, "%s: no plugin description!\n", __FUNCTION__);
+      fprintf (stderr, _("%s: no plugin description!\n"), __FUNCTION__);
       return;
     }
   
@@ -118,7 +115,6 @@ void
 new_cb (GtkWidget * widget, gpointer user_data)
 {
   ui_t * ui;
-  ctrlmsg_t ctrlmsg;
   
   ui = (ui_t *) user_data;
   
@@ -128,58 +124,36 @@ new_cb (GtkWidget * widget, gpointer user_data)
 }
 
 #ifdef HAVE_XML
-static const char *
-get_filename ()
-{
-  static GtkWidget * dialog = NULL;
-  static char * file = NULL;
-  int response;
-  
-  if (!dialog)
-    dialog = gtk_file_selection_new (_("Select file"));
-  
-  gtk_widget_show (dialog);
-  gtk_file_selection_complete (GTK_FILE_SELECTION (dialog), "*.rack");
-  response = gtk_dialog_run (GTK_DIALOG (dialog));
-  gtk_widget_hide (dialog);
-  
-  if (response != GTK_RESPONSE_OK)
-    return NULL;
-
-  if (file)
-    g_free (file);
-  
-  file = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog)));
-  return file;
-}
-
-
 void
 open_cb (GtkButton * button, gpointer user_data)
 {
-  const char * filename;
-  ui_t * ui;
-  int err;
-  
-  ui = (ui_t *) user_data;
-  
-  filename = get_filename ();
-  
-  if (!filename)
-    return;
-  
-  err = ui_open_file (ui, filename);
-  
-  if (!err)
-    ui_set_filename (ui, filename);
+  ui_t * ui = (ui_t *) user_data;
+  GtkWidget * dialog;
+
+  dialog = gtk_file_chooser_dialog_new (_("Open File"), NULL,
+                                        GTK_FILE_CHOOSER_ACTION_OPEN,
+                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                        NULL);
+
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+    {
+      char * filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+
+      if (!ui_open_file (ui, filename))
+        ui_set_filename (ui, filename);
+
+      g_free (filename);
+    }
+
+  gtk_widget_destroy (dialog);
 }
- 
+
+
 void
 save_cb (GtkButton * button, gpointer user_data)
 {
-  ui_t * ui;
-  
-  ui = (ui_t *) user_data;
+  ui_t * ui = (ui_t *) user_data;
   
   if (!ui->filename)
     {
@@ -191,38 +165,42 @@ save_cb (GtkButton * button, gpointer user_data)
 }
 
 
- 
 void
 save_as_cb (GtkButton * button, gpointer user_data)
 {
-  const char * filename;
-  ui_t * ui;
-  int err;
-  
-  ui = (ui_t *) user_data;
-  
-  filename = get_filename (ui->jack_rack);
-  
-  if (!filename)
-    return;
-  
-  err = ui_save_file (ui, filename);
-  
-  if (!err)
-    ui_set_filename (ui, filename);
+  ui_t * ui = (ui_t *) user_data;
+  GtkWidget * dialog;
+
+  dialog = gtk_file_chooser_dialog_new (_("Save File"), NULL,
+                                        GTK_FILE_CHOOSER_ACTION_SAVE,
+                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                        GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+                                        NULL);
+
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+    {
+      char * filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+
+      if (!ui_save_file (ui, filename))
+        ui_set_filename (ui, filename);
+
+      g_free (filename);
+    }
+
+  gtk_widget_destroy (dialog);
 }
 #endif /* HAVE_XML */
  
-#ifdef HAVE_LADCCA
+#ifdef HAVE_LASH
 void
-cca_save_cb (GtkButton * button, gpointer user_data)
+lash_save_cb (GtkButton * button, gpointer user_data)
 {
-  cca_event_t * event;
+  lash_event_t * event;
   
-  event = cca_event_new_with_type (CCA_Save);
-  cca_send_event (global_cca_client, event);
+  event = lash_event_new_with_type (LASH_Save);
+  lash_send_event (global_lash_client, event);
 }
-#endif /* HAVE_LADCCA */
+#endif /* HAVE_LASH */
 
 
 void
@@ -238,6 +216,7 @@ quit_cb (GtkButton * button, gpointer user_data)
   if (ui->shutdown)
     {
       gtk_main_quit ();
+      process_info_destroy (ui->procinfo);
       return;
     }
   
@@ -260,14 +239,11 @@ window_destroy_cb (GtkWidget *widget,
 void
 about_cb (GtkWidget * widget, gpointer user_data)
 {
-  GtkWidget * dialog;
-  const char * authors[] = { "Bob Ham <node@users.sourceforge.net>", NULL };
-  const char * documenters[] = { NULL };
-  GtkWidget * url;
+  const char * authors[] = { "Bob Ham <node@users.sourceforge.net>", "Leslie P. Polzer <leslie.polzer@gmx.net>", "Adam Sampson <ats@offog.org>", NULL };
   gchar * logo_file;
   GdkPixbuf * logo;
   GError * err = NULL;
-  
+
   logo_file = g_strdup_printf ("%s/%s", PKGDATADIR, JACK_RACK_LOGO_FILE);
   logo = gdk_pixbuf_new_from_file (logo_file, &err);
   if (err)
@@ -277,44 +253,36 @@ about_cb (GtkWidget * widget, gpointer user_data)
       g_error_free (err);
     }
   g_free (logo_file);
- 
-  dialog = gnome_about_new (PACKAGE_NAME,
-                              PACKAGE_VERSION,
-                              "Copyright (C) 2002, 2003 Robert Ham <node@users.sourceforge.net>",
-                              _("An LADSPA effects rack for the JACK audio API"),
-                              authors,
-                              documenters,
-                              "",
-                              logo);
-  
-  url = gnome_href_new (JACK_RACK_URL, JACK_RACK_URL);
-  gtk_widget_show (url);
-  GTK_WIDGET_UNSET_FLAGS (url, GTK_CAN_FOCUS);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), url, FALSE, FALSE, 0);
-  
-  gtk_dialog_run (GTK_DIALOG (dialog));
+
+  gtk_show_about_dialog (NULL,
+                         "authors", authors,
+                         "comments", _("A LADSPA effects rack for the JACK audio API"),
+                         "copyright", COPYRIGHT_MESSAGE,
+                         "logo", logo,
+                         "name", PACKAGE_NAME,
+                         "translator-credits", _("__TRANSLATOR_CREDITS"),
+                         "version", PACKAGE_VERSION,
+                         "website", JACK_RACK_URL,
+                         NULL);
+
 }
 #endif /* HAVE_GNOME */
 
 
 #ifdef HAVE_ALSA
 void
-midi_cb           (GtkWidget * button, gpointer user_data)
+midi_cb (GtkWidget * button, gpointer user_data)
 {
   ui_t * ui = user_data;
   gtk_widget_show (ui->midi_window->window);
 }
-#endif /* HAVE_ALSA
+#endif /* HAVE_ALSA */
 
 
-
-
-
-/** callback for plugin menu buttons (Add and the plugin slots' change buttons) */
+/** callback for plugin menu buttons ("Add" and the plugin slots' change buttons) */
 gint
 plugin_button_cb (GtkWidget *widget, GdkEvent *event)
 {
-
   if (event->type == GDK_BUTTON_PRESS)
     {
       GdkEventButton * bevent = (GdkEventButton *) event;
@@ -329,50 +297,59 @@ plugin_button_cb (GtkWidget *widget, GdkEvent *event)
   return FALSE;
 }
 
-#ifdef HAVE_LADCCA
-static int
-cca_idle (ui_t * ui, cca_client_t * client)
-{
-  cca_event_t * event;
 
-  while ( (event = cca_get_event (client)) )
+void
+plugin_add_button_cb (GtkWidget *widget, gpointer user_data)
+{
+  gtk_menu_popup (GTK_MENU (user_data), NULL, NULL, NULL, NULL,
+                  0, gtk_get_current_event_time ());
+}
+
+
+#ifdef HAVE_LASH
+static int
+lash_idle (ui_t * ui, lash_client_t * client)
+{
+  lash_event_t * event;
+
+  while ( (event = lash_get_event (client)) )
     {
-      switch (cca_event_get_type (event))
-	{
-	case CCA_Save_File:
-	  ui_save_file (ui, cca_get_fqn (cca_event_get_string (event), "jack_rack.rack"));
-	  cca_send_event (global_cca_client, event);
-	  break;
-	case CCA_Restore_File:
-	  ui_open_file (ui, cca_get_fqn (cca_event_get_string (event), "jack_rack.rack"));
-	  cca_send_event (global_cca_client, event);
-	  break;
-	case CCA_Quit:
-	  quit_cb (NULL, ui);
-	  cca_event_destroy (event);
-	  break;
-	case CCA_Server_Lost:
-	  printf ("server lost\n");
-	  printf (_("LADCCA server disconnected\n"));
-	  gtk_widget_set_sensitive (ui->cca_save, FALSE);
-	  gtk_widget_set_sensitive (ui->cca_save_menu_item, FALSE);
-	  cca_event_destroy (event);
-	  return 0;
-	  break;
-	default:
-	  fprintf (stderr, "Recieved LADCCA event of unknown type %d\n", cca_event_get_type (event));
-	  cca_event_destroy (event);
-	  break;
-	}
+      switch (lash_event_get_type (event))
+        {
+        case LASH_Save_File:
+          ui_save_file (ui, lash_get_fqn (lash_event_get_string (event), "jack_rack.rack"));
+          lash_send_event (global_lash_client, event);
+          break;
+        case LASH_Restore_File:
+          ui_open_file (ui, lash_get_fqn (lash_event_get_string (event), "jack_rack.rack"));
+          lash_send_event (global_lash_client, event);
+          break;
+        case LASH_Quit:
+          quit_cb (NULL, ui);
+          lash_event_destroy (event);
+          break;
+        case LASH_Server_Lost:
+          printf ("server lost\n");
+          printf (_("LASH server disconnected\n"));
+          gtk_widget_set_sensitive (GTK_WIDGET (ui->lash_save), FALSE);
+          gtk_widget_set_sensitive (GTK_WIDGET (ui->lash_save_menu_item), FALSE);
+          lash_event_destroy (event);
+          return 0;
+          break;
+        default:
+          fprintf (stderr, _("Received LASH event of unknown type %d\n"), lash_event_get_type (event));
+          lash_event_destroy (event);
+          break;
+        }
     }
 
-  if (!cca_enabled (client))
+  if (!lash_enabled (client))
     return 0;
 
 
   return 1;
 }
-#endif /* HAVE_LADCCA */
+#endif /* HAVE_LASH */
 
 
 #ifdef HAVE_ALSA
@@ -403,8 +380,6 @@ ui_set_port_value (ui_t *ui, midi_control_t *midi_control, LADSPA_Data value)
 static void
 ui_set_wet_dry_value (ui_t *ui, midi_control_t *midi_control, LADSPA_Data value)
 {
-  GtkWidget * widget;
-  
   if (!midi_control_get_locked (midi_control))
     settings_set_wet_dry_value (midi_control->plugin_slot->settings,
                                 midi_control->control.wet_dry.channel,
@@ -442,48 +417,146 @@ midi_idle (ui_t * ui)
           midi_control_destroy (ctrlmsg.data.midi.midi_control);
           break;
         case CTRLMSG_MIDI_CTRL:
-          if (ctrlmsg.data.midi.midi_control->ladspa_control)
-            ui_set_port_value (ui, ctrlmsg.data.midi.midi_control, ctrlmsg.data.midi.value);
-          else
-            ui_set_wet_dry_value (ui, ctrlmsg.data.midi.midi_control, ctrlmsg.data.midi.value);
+          if (ctrlmsg.data.midi.midi_control->ctrl_type == LADSPA_CONTROL)
+            ui_set_port_value (ui, ctrlmsg.data.midi.midi_control, 
+			       ctrlmsg.data.midi.value);
+          else if(ctrlmsg.data.midi.midi_control->ctrl_type == WET_DRY_CONTROL)
+            ui_set_wet_dry_value (ui, ctrlmsg.data.midi.midi_control, 
+				  ctrlmsg.data.midi.value);
+          break;
+	    case CTRLMSG_MIDI_ABLE:
+	      gtk_toggle_button_set_active (
+		          GTK_TOGGLE_BUTTON(ctrlmsg.data.ablise.plugin_slot->enable),
+		          ctrlmsg.data.ablise.enable);
+	      break; 
+        default:
           break;
         }
     }
 }  
 #endif /* HAVE_ALSA */
 
+
+struct reconnect_data
+{
+        gboolean active;
+        GtkWidget* dialog;
+        
+        ui_t* ui;
+};
+
+
+/**
+ * Attempt to reconnect to JACK.
+ *
+ * @param data Argument of type 'struct reconnect_data'
+ * 
+ * @return FALSE on success (this tells Glib to delete the timeout)
+ */
+static gboolean
+reconnect_cb ( gpointer data )
+{
+        struct reconnect_data*  rcdata = (struct reconnect_data*)data;
+
+        process_info_t*         procinfo = rcdata->ui->procinfo;
+        
+        if ( rcdata->active == FALSE )
+                return FALSE;
+
+        /* reconnect */
+        procinfo = process_info_new (rcdata->ui, 2);
+        
+        if (!procinfo)
+                /* attempt not successful, maintain callback */
+                return TRUE;
+        
+        /* attempt successful */
+        
+        gtk_widget_hide (rcdata->dialog);
+        rcdata->ui->shutdown = FALSE;
+
+        return FALSE;
+}
+
+
+void
+setup_reconnect ( gpointer data )
+{
+        static GtkWidget* dialog = NULL;
+        static int active = TRUE;
+        static struct reconnect_data rcdata;
+	guint tid;
+	gint answer;
+	GtkWidget *rcnotice;
+
+        ui_t* ui = (ui_t*)data;
+        
+        rcdata.active = TRUE;
+        rcdata.ui = ui;
+        
+        if ( dialog == NULL )
+              dialog = gtk_dialog_new_with_buttons (
+                                _("Connecting to JACK..."),
+                                NULL, GTK_DIALOG_MODAL,
+                                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                NULL );
+        rcnotice = gtk_label_new (_("Connecting to JACK server..."));
+        gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), rcnotice);
+        gtk_container_set_border_width (GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 50);
+        gtk_widget_set_size_request (GTK_DIALOG(dialog)->vbox, 250, 100); 
+        gtk_widget_show_all (GTK_DIALOG(dialog)->vbox);
+        rcdata.dialog = dialog;
+        
+        tid = g_timeout_add (1000, &reconnect_cb, (gpointer)&rcdata);
+        
+        answer = gtk_dialog_run (GTK_DIALOG(dialog));
+        
+        if (answer == GTK_RESPONSE_CANCEL)
+        {
+                /* FIXME show notice */
+                rcdata.active = FALSE;
+
+                if (ui->main_window == NULL)
+                {
+                        ui_set_state (ui, STATE_QUITTING);
+                        return;
+                }
+                
+                gtk_widget_set_sensitive (ui->plugin_box, FALSE);
+                gtk_widget_set_sensitive (GTK_WIDGET (ui->add), FALSE);
+                gtk_widget_set_sensitive (ui->add_menuitem, FALSE);
+                gtk_widget_set_sensitive (GTK_WIDGET (ui->channels), FALSE);
+                gtk_widget_set_sensitive (ui->channels_menuitem, FALSE);
+                gtk_widget_set_sensitive (GTK_WIDGET (ui->new), FALSE);
+                gtk_widget_set_sensitive (ui->new_menuitem, FALSE);
+#ifdef HAVE_ALSA
+                gtk_widget_set_sensitive (ui->midi_menuitem, FALSE);
+                gtk_widget_set_sensitive (ui->midi_window->main_box, FALSE);
+#endif
+#ifdef HAVE_XML
+                gtk_widget_set_sensitive (GTK_WIDGET (ui->open), FALSE);
+                gtk_widget_set_sensitive (ui->open_menuitem, FALSE);
+#endif
+        }
+        
+        active = FALSE;
+        gtk_widget_hide (dialog);
+}
+
+
 static void
 ui_check_kicked (ui_t * ui)
 {
-  static gboolean shown_shutdown_msg = FALSE;
+        static gboolean shown_shutdown_msg = FALSE;
   
-  if (ui->shutdown &&
-      !shown_shutdown_msg &&
-      ui_get_state (ui) != STATE_QUITTING)
-    {
-      ui_display_error (ui, _("%s\n\n%s%s"),
-                        _("JACK client thread shut down by server"),
-                        _("JACK, the bastard, kicked us out.  "),
-                        _("You'll have to restart I'm afraid.  Sorry."));
-    
-      gtk_widget_set_sensitive (ui->plugin_box, FALSE);
-      gtk_widget_set_sensitive (ui->add, FALSE);
-      gtk_widget_set_sensitive (ui->add_menuitem, FALSE);
-      gtk_widget_set_sensitive (ui->channels, FALSE);
-      gtk_widget_set_sensitive (ui->channels_menuitem, FALSE);
-      gtk_widget_set_sensitive (ui->new, FALSE);
-      gtk_widget_set_sensitive (ui->new_menuitem, FALSE);
-#ifdef HAVE_ALSA
-      gtk_widget_set_sensitive (ui->midi_menuitem, FALSE);
-      gtk_widget_set_sensitive (ui->midi_window->main_box, FALSE);
-#endif
-#ifdef HAVE_XML
-      gtk_widget_set_sensitive (ui->open, FALSE);
-      gtk_widget_set_sensitive (ui->open_menuitem, FALSE);
-#endif
+        if (ui->shutdown && !shown_shutdown_msg
+            && ui_get_state (ui) != STATE_QUITTING)
+        {
+                setup_reconnect (ui);
+                shown_shutdown_msg = TRUE;
+        }
 
-      shown_shutdown_msg = TRUE;
-    }
+        return;
 }
 
 
@@ -494,10 +567,9 @@ idle_cb (gpointer data)
   ctrlmsg_t ctrlmsg;
   ui_t * ui;
   jack_rack_t * jack_rack;
-  gboolean enabled;
-#ifdef HAVE_LADCCA
-  static int call_cca_idle = 1;
-#endif /* HAVE_LADCCA */
+#ifdef HAVE_LASH
+  static int call_lash_idle = 1;
+#endif /* HAVE_LASH */
 
   ui = (ui_t *) data;
   jack_rack = ui->jack_rack;
@@ -557,14 +629,17 @@ idle_cb (gpointer data)
 
       case CTRLMSG_QUIT:
         gtk_main_quit ();
+        process_info_destroy (ui->procinfo);
         break;
         
+      default:
+        break;
       }
     }
 
-#ifdef HAVE_LADCCA
-  if (call_cca_idle)
-    call_cca_idle = cca_idle (ui, global_cca_client);
+#ifdef HAVE_LASH
+  if (call_lash_idle)
+    call_lash_idle = lash_idle (ui, global_lash_client);
 #endif
   
   usleep (1000);
@@ -575,7 +650,7 @@ idle_cb (gpointer data)
 void
 jack_shutdown_cb (void * data)
 {
-  ui_t * ui = data;
+  ui_t * ui = (ui_t*)data;
   
   ui->shutdown = TRUE;
 }
